package controller;

import gui.ComponentComposor;

import java.awt.event.ActionEvent;

import javax.swing.ActionMap;
import javax.swing.event.ListSelectionEvent;
import javax.swing.table.TableModel;

import model.LibraryModel;
import model.api.Playlist;
import model.api.Song;
import model.impl.LibraryManager;


/*
 * Created on 2004. 1. 29.
 *
 */

/**
 * @author hyoki
 *
 */
public class ActionController {
	
	private LibraryManager libraryManager = null;
	private PlayerController playerController = null;
	private ComponentComposor composer = null;
	private ActionMap actionMap = null;
	/**
	 * @param string
	 */
	public ActionController(LibraryManager libraryManager) {
		this.libraryManager = libraryManager;
		playerController = new PlayerController(this);
	}
	/**
	 * @return
	 */
	public ActionMap getActionMap() {		
		if (actionMap == null) {
			actionMap = new ActionFactory(this).createActionMap();			
		}
		return actionMap;
	}
	/**
	 * @param composor
	 */
	public void setComponentComposor(ComponentComposor composor) {
		this.composer = composor;	
	}

	public void newList(ActionEvent event){	
		libraryManager.getLibraries()[0].createPlaylist("untitle playlist");
	}
	public void newListSelected(ActionEvent event){	
		System.out.println(event);
	}
	public void newSmartList(ActionEvent event){	
		System.out.println(event);
	}
	public void addFile(ActionEvent event){	
		System.out.println(event);
	}
	public void addFolder(ActionEvent event){	
		System.out.println(event);
	}
	public void closeWindow(ActionEvent event){	
		System.exit(0);
	}
	public void importList(ActionEvent event){	
		System.out.println(event);
	}
	public void exportList(ActionEvent event){	
		System.out.println(event);
	}
	public void exportLib(ActionEvent event){	
		System.out.println(event);
	}
	public void getInfo(ActionEvent event){	
		System.out.println(event);
	}
	public void editSmartList(ActionEvent event){	
		System.out.println(event);
	}	
	public void showSongFile(ActionEvent event){	
		System.out.println(event);
	}
	public void showCurrentSong(ActionEvent event){	
		System.out.println(event);
	}
	public void burnCD(ActionEvent event){	
		System.out.println(event);
	}
	public void updateIpod(ActionEvent event){	
		System.out.println(event);
	}
	public void exit(ActionEvent event){	
		System.exit(0);
	}	
	public void undo(ActionEvent event){	
			System.out.println(event);
	}
	public void cut(ActionEvent event){	
			System.out.println(event);
	}
	public void copy(ActionEvent event){	
			System.out.println(event);
	}
	public void paste(ActionEvent event){	
			System.out.println(event);
	}
	public void clear(ActionEvent event){	
		System.out.println(event);
	}
	public void selectAll(ActionEvent event){	
		System.out.println(event);
	}
	public void selectNone(ActionEvent event){	
		System.out.println(event);
	}
	public void browser(ActionEvent event){
		javax.swing.Action action = ((javax.swing.AbstractButton)event.getSource()).getAction();
		if(action.getValue(ActionFactory.BROWSER).equals(Boolean.TRUE)){
			composer.browserOff();
			action.putValue(ActionFactory.BROWSER,Boolean.FALSE);
		}else{
			composer.browserOn();
			action.putValue(ActionFactory.BROWSER,Boolean.TRUE);
		}
	}
	public void artwork(ActionEvent event){
		javax.swing.Action action = ((javax.swing.AbstractButton)event.getSource()).getAction();
		if(action.getValue(ActionFactory.ARTWORK).equals(Boolean.TRUE)){
			composer.artworkOff();
			action.putValue(ActionFactory.ARTWORK,Boolean.FALSE);
		}else{
			composer.artworkOn(new javax.swing.ImageIcon().getImage());
			action.putValue(ActionFactory.ARTWORK,Boolean.TRUE);
		}
	}
	public void viewOption(ActionEvent event){	
		System.out.println(event);
	}	
	public void preference(ActionEvent event){
		System.out.println(event);
	}
	

	private Playlist currentPlaylist = null;
	private int playedSongIndex = 0;
	public void play(ActionEvent eventvent){
		Song song = null;
		try {
			playedSongIndex = composer.getSongTable().getSelectedRow();
			song = getSongFromTable(playedSongIndex);
		} catch (ArrayIndexOutOfBoundsException e) {
			playedSongIndex = 0;
			song = getSongFromTable(0);
		}
		tryPlaySong(song);
	}
	private void tryPlaySong(Song song) {
		try {			
			playerController.play(song);
			composer.readyPause();
			composer.previosSong(true);
			composer.nextSong(true);			
			composer.setCurruntSong(song);		
			this.currentPlaylist =((model.LibraryModel)composer.getPlaylistTable().getModel()).getList(composer.getPlaylistTable().getSelectedRow());
		} catch (PlayerPauseException e1) {
			pause(null);
		} catch (PlayerStopException e) {
			stop(null);
		}
	}
	private Song getSongFromTable(int index)throws ArrayIndexOutOfBoundsException {
		return ((model.PlaylistModel)composer.getSongTable().getModel()).getSong(index);
	}

	public void stop(ActionEvent event){		
		playerController.Stop();
		composer.clearCurrentSong();
		composer.readyPlay();
		composer.previosSong(false);
		composer.nextSong(false);
	}
	public void pause(ActionEvent event){
		playerController.pause();
		composer.readyPlay();
	}	
	public void nextSong(ActionEvent event){
		Song song = null;
		try {
			song = getSongFromTable(++playedSongIndex);
			tryPlaySong(song);
		} catch (ArrayIndexOutOfBoundsException e) {
			stop(null);			
		}finally{
			checkBound();
		}
	}
	public void previosSong(ActionEvent event){
		Song song = null;
		try {
			song = getSongFromTable(--playedSongIndex);
			tryPlaySong(song);
		} catch (ArrayIndexOutOfBoundsException e) {
			stop(null);		
		}finally{
			checkBound();
		}
	}
	private void checkBound() {
		composer.nextSong(playedSongIndex < currentPlaylist.getSongSize());
		composer.previosSong(playedSongIndex > 0);
	}	
	public void shuffle(ActionEvent event){
		System.out.println(event);
	}
	public void repeatOff(ActionEvent event){
		System.out.println(event);
	}
	public void repeatAll(ActionEvent event){	
		System.out.println(event);
	}
	public void volumeUp(ActionEvent event){
		int gain = composer.getVolumeSlider().getValue();
		gain+=5;
		composer.getVolumeSlider().setValue(gain);
		composer.getVolumeSlider().setValueIsAdjusting(false);
	}
	public void volumeDown(ActionEvent event){
		int gain = composer.getVolumeSlider().getValue();
		gain-=5;
		composer.getVolumeSlider().setValue(gain);
		composer.getVolumeSlider().setValueIsAdjusting(false);
	}	
	public void mute(ActionEvent event){
		javax.swing.Action action = ((javax.swing.AbstractButton)event.getSource()).getAction();		
		if(action.getValue(ActionFactory.MUTE) == null){
			action.putValue(ActionFactory.MUTE,composer.getVolumeSlider().getValue()+"");
			composer.getVolumeSlider().setValue(0);
		}else{				
			int gain = Integer.parseInt((String) action.getValue(ActionFactory.MUTE));
			action.putValue(ActionFactory.MUTE,null);
			composer.getVolumeSlider().setValue(gain);
		}
		composer.getVolumeSlider().setValueIsAdjusting(false);
	}
	public void about(ActionEvent event){
		composer.showAbout(); 
	}	


	/**
	 * @param e
	 */
	public void changeSongTable(ListSelectionEvent e) {
		javax.swing.DefaultListSelectionModel select = (javax.swing.DefaultListSelectionModel) e.getSource();		
		Playlist playlist = ((LibraryModel) composer.getPlaylistTable().getModel()).getList(select.getAnchorSelectionIndex());
		model.PlaylistModel model = new model.PlaylistModel(playlist);
		composer.getSongTable().setModel(model);
		if(playerController.isPlaying()){
			if(currentPlaylist.equals(playlist)){				
				composer.readyPause();
			}else{
				composer.readyStop();
			}
		} else{
			stop(null);
		}
	}
	/**
	 * @return
	 */
	public TableModel getLibraryModel() {
		return new model.LibraryModel(libraryManager);
	}
	/**
	 * @param source
	 */
	public void updateVolume(javax.swing.JSlider source) {
		playerController.setGain(source.getValue(),source.getMaximum());
	}
	/**
	 * @param cursor
	 * @param total
	 */
	public void updateCursor(int cursor, int total) {
		composer.setTimeData(cursor,total);
	}
	/**
	 * @param string
	 * @param i
	 */
	public void setPlaylistName(String string, int i) {
		LibraryModel model = (LibraryModel) composer.getPlaylistTable().getModel();
		model.getList(i).setName(string);
	}	
}
