package daapLib;


import java.util.Date;
import java.util.HashMap;

public class DaapParser {
    private char[] HEX_CHARS = {'0', '1', '2', '3', '4', '5', '6', '7', '8',
                                '9', 'a', 'b', 'c', 'd', 'e', 'f'};
    HashMap values;

    public DaapParser() {
    
    }
    
    public void parse(byte[] data) {
        try {
            DaapContentCodes contentCodes = new DaapContentCodes();
            parse(data, contentCodes);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void parse(byte[] data, DaapContentCodes contentCodes) throws DaapParserException {
        values = new HashMap();
        int position = 0;
        
        while (position < data.length) {
            byte[] tagData = byteArraySlicer(data, position, position+4);
            position += 4;
            
            String tag = getTagName(tagData);

            int i=0;
            String tagvalue = new String();
            for (i=position;i<position+4;i++) {
                int b = data[i] & 0xff;
                tagvalue += HEX_CHARS[b>>4] + "" + HEX_CHARS[b&0x0f];
            }
            position=i;
            int length = Integer.decode("0x"+tagvalue).intValue();
            
            int contentCodeType = contentCodes.getContentCodeTypeFor(tag);
            Object tagobject = new Object();
            switch (contentCodeType) {
                case 1:		// Boolean, false if no value, true if value exists
                case 3:		// Integer
                case 5:		//
                    tagvalue = new String();
                    for (i=position;i<position+length;i++) {
                        int b = data[i]&0xff;
                        tagvalue += HEX_CHARS[b>>4] + "" + HEX_CHARS[b&0x0f];
                    }
                    position=i;
                    tagobject = Integer.decode("0x"+tagvalue);
                    break;
                case 10:	// Date
                    tagvalue = new String();
                    for (i=position;i<position+length;i++) {
                        int b = data[i]&0xff;
                        tagvalue += HEX_CHARS[b>>4] + "" + HEX_CHARS[b&0x0f];
                    }
                    position=i;
                    int epoch = Integer.decode("0x"+tagvalue).intValue();
                    tagobject = new Date((long)epoch*1000).toString();
                    break;
                case 7:		// Hex String
                case 11:	// Version (8 bytes, first 4 = Major, last 4 = Minor)
                    tagvalue = new String();
                    for (i=position;i<position+length;i++) {
                        int b = data[i]&0xff;
                        tagvalue += HEX_CHARS[b>>4] + "" + HEX_CHARS[b&0x0f];
                    }
                    position=i;
                    tagobject = tagvalue;
                    break;
                case 9:		// String
                    tagvalue = new String();
                    for (i=position;i<position+length;i++) {
                        int value;
                        if (((data[i]&0xff)&0x80) == 0) {
                            tagvalue += (char)data[i];
                        } else if (((data[i]&0xff)&0xe0) == 0xc0) {
                            value = (data[i++] & 0x1f) << 6;
                            value |= (data[i] & 0x3f);
                            tagvalue += (char)value;
                        } else if (((data[i]&0xff)&0xf0) == 0xe0) {
                            value = (data[i++] & 0x0F) << 12;
                            value |= (data[i++] & 0x3F) << 6;
                            value |= (data[i] & 0x3F);
                            tagvalue += (char)value;
                        } else if (((data[i]&0xff)&0xf8) == 0xf0) {
                            value = (data[i++] & 0x07) << 18;
                            value |= (data[i++] & 0x3f) << 12;
                            value |= (data[i++] & 0x3f) << 6;
                            value |= (data[i] & 0x3f);
                            tagvalue += (char)value;
                        }
                    }
                    position=i;
                    tagobject = tagvalue;
                    break;
                case 12:	//
                    tagobject = new Integer(length);
                    break;
                default:	// we don't know how to handle this!
                    throw new DaapParserException("Unknown content code type '" +
                                              contentCodeType + "' found at " + 
                                              "position " + position);  
            }
            values.put(tag, tagobject);
        }        
    }

    public String getTagName(byte[] data) {
        String tag = new String();
        tag += (char)data[0];
        tag += (char)data[1];
        tag += (char)data[2];
        tag += (char)data[3];
        return(tag);
    }
    
    public byte[] byteArraySlicer(byte[] data, int start, int end) {
        byte[] returnValue = new byte[end - start];
        
        int curPos = 0;
        for (int i=start; i<end; i++) {
            returnValue[curPos] = data[i];
            curPos++;
        }
        return(returnValue);
    }
    
    public Object getObjectForKey(String key) {
        return(values.get(key));
    }

    public String toString() {
        return(values.toString());
    }
}
