package daapLib;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.tree.TreeNode;

public class DaapPlaylists implements TreeNode{
    private int databaseplaylists = -1;
    private int status = -1;
    private int updatetype = -1;
    private int specifiedtotalcount = -1;
    private int returnedcount = -1;
    private int listing = -1;
    ArrayList playlists;
    private DaapShared parent = null;
    
    public DaapPlaylists(byte[] data, DaapShared parent) {
        this.parent = parent;
        int startOfPlaylist = 0;
        while (startOfPlaylist < data.length) {
            if (data[startOfPlaylist] == 'm' && data[startOfPlaylist+1] == 'l' &&
                data[startOfPlaylist+2] == 'i' && data[startOfPlaylist+3] == 't') {
                
                break;
            } else {
                startOfPlaylist++;
            }
        }
        
        byte[] headerData = new byte[startOfPlaylist];
        System.arraycopy(data, 0, headerData, 0, startOfPlaylist);
        DaapParser parser = new DaapParser();
        parser.parse(headerData);
        
        if (parser.getObjectForKey("aply") != null)
            databaseplaylists = ((Integer)parser.getObjectForKey("aply")).intValue();
        if (parser.getObjectForKey("mstt") != null)
            status = ((Integer)parser.getObjectForKey("mstt")).intValue();
        if (parser.getObjectForKey("muty") != null)
            updatetype = ((Integer)parser.getObjectForKey("muty")).intValue();
        if (parser.getObjectForKey("mtco") != null)
            specifiedtotalcount = ((Integer)parser.getObjectForKey("mtco")).intValue();
        if (parser.getObjectForKey("mrco") != null)
            returnedcount = ((Integer)parser.getObjectForKey("mrco")).intValue();
        if (parser.getObjectForKey("mlcl") != null)
            listing = ((Integer)parser.getObjectForKey("mlcl")).intValue();
        returnedcount=Math.max(0,returnedcount);
        playlists = new ArrayList(returnedcount);
        
        int endOfPlaylist = startOfPlaylist + 1;
        while (endOfPlaylist < data.length) {
            if (data[endOfPlaylist] == 'm' && data[endOfPlaylist+1] == 'l' &&
                data[endOfPlaylist+2] == 'i' && data[endOfPlaylist+3] == 't') {
                
                int payloadLength = endOfPlaylist - startOfPlaylist;
                byte[] playlistData = new byte[payloadLength];
                System.arraycopy(data, startOfPlaylist, playlistData, 0, payloadLength);
                DaapPlaylist playlist = new DaapPlaylist(playlistData, parent);
                playlists.add(playlist);
                startOfPlaylist = endOfPlaylist;
                endOfPlaylist += 4;
            }
            endOfPlaylist++;
        }
        // And once more, for the final item
        int payloadLength = data.length - startOfPlaylist;
        byte[] playlistData = new byte[payloadLength];
        System.arraycopy(data, startOfPlaylist, playlistData, 0, payloadLength);
        DaapPlaylist playlist = new DaapPlaylist(playlistData, parent);
        playlists.add(playlist);
        
        if (playlists.size() != returnedcount) {
            // Find a better way to handle this error
        }
    }
    
    public int getDatabasePlaylists() {
        return(databaseplaylists);
    }
    
    public int getStatus() {
        return(status);
    }
    
    public int getUpdateType() {
        return(updatetype);
    }
    
    public int getSpecifiedTotalCount() {
        return(specifiedtotalcount);
    }
    
    public int getReturnedCount() {
        return(returnedcount);
    }
    
    public int getListing() {
        return(listing);
    }
    
    public int count() {
        return(playlists.size());
    }
    
    public DaapPlaylist getPlaylistAtIndex(int index) {
        return((DaapPlaylist)playlists.get(index));
    }
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("\nPlaylists");
        buf.append("\ndaap.databaseplaylists=" + databaseplaylists);
        buf.append("\ndmap.status=" + status);
        buf.append("\ndmap.updatetype=" + updatetype);
        buf.append("\ndmap.specifiedtotalcount=" + specifiedtotalcount);
        buf.append("\ndmap.returnedcount=" + returnedcount);
        buf.append("\ndmap.listing=" + listing);
        buf.append("\nplaylists.size()=" + playlists.size());
        return(buf.toString());
    }

    public TreeNode getChildAt(int i) {
        return (DaapPlaylist)this.playlists.get(i);
    }

    public int getChildCount() {
        return this.playlists.size();
    }

    public TreeNode getParent() {
        return (TreeNode) parent;
    }

    public int getIndex(TreeNode node) {
        return playlists.indexOf(node);
    }

    public boolean getAllowsChildren() {
        return true;
    }

    public boolean isLeaf() {
        return this.getChildCount()==0;
    }

    public Enumeration children() {
        return new Vector(playlists).elements();
    }

    public void setParent(DaapShared parent){
        this.parent = parent;
        for(int i=0;i<playlists.size(); i++){
			((DaapPlaylist)playlists.get(i)).setParent((TreeNode) parent);
        }
    }
}
