package daapLib;


import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.net.HttpURLConnection;

import daapLib.api.DaapRemoteSong;
import daapLib.network.NetworkTransferThread;

//import javax.swing.tree.TreeNode;

public class DaapServer extends Thread {

    private DaapServerInfoResponse serverInfoResponse;
    private DaapLoginResponse loginResponse;
	private DaapUpdateResponse updateResponse;
	private DaapDatabasesResponse databasesResponse;
	private DaapPlaylists playlists;
	private DaapSongs songs;
	private DaapSongs playlist_songs;

	private String hostname;
	private String password;

	private String statusMessage;
	private boolean completed;
	private boolean errorOccured;
	private int errorCode;
	private int httpStatus;
	private NetworkTransferThread network;
	private DaapShared parent = null;
	private boolean getAll = false;
	
    
    public DaapServer(String hostname, String password, DaapShared parent) {
        this.parent = parent;
        this.hostname = hostname;
        this.password = password;
        statusMessage = "Idle";
        completed = false;
        errorOccured = false;

    }
    public String getDownloadMeta(){
		String downloadMetadata = "dmap.itemid,dmap.itemname,dmap.itemkind,dmap.persistentid,daap.songformat,";
        //if (userDefaults.booleanForKey("Artist"))
            downloadMetadata += "daap.songartist,";
        //if (userDefaults.booleanForKey("Album"))
            downloadMetadata += "daap.songalbum,";
        if (booleanForKey("Beats Per Minute"))
            downloadMetadata += "daap.songbeatsperminute,";
        if (booleanForKey("Bit Rate"))
            downloadMetadata += "daap.songbitrate,";
        if (booleanForKey("Comment"))
            downloadMetadata += "daap.songcomment,";
        if (booleanForKey("Composer"))
            downloadMetadata += "daap.songcomposer,";
        if (booleanForKey("Date Added"))
            downloadMetadata += "daap.songdateadded,";
        if (booleanForKey("Date Modified"))
            downloadMetadata += "daap.songdatemodified,";
        if (booleanForKey("Disc"))
            downloadMetadata += "daap.songdisccount,daap.songdiscnumber,";
        //if (userDefaults.booleanForKey("Genre"))
            downloadMetadata += "daap.songgenre,";
        if (booleanForKey("Kind"))
            downloadMetadata += "daap.songdescription,";
        //if (booleanForKey("Rating"))
            downloadMetadata += "daap.songuserrating,";
        if (booleanForKey("Sample Rate"))
            downloadMetadata += "daap.songsamplerate,";
        if (booleanForKey("Size"))
            downloadMetadata += "daap.songsize,";
        //if (userDefaults.booleanForKey("Time"))
            downloadMetadata += "daap.songtime,";
        if (booleanForKey("Track"))
            downloadMetadata += "daap.songtrackcount,daap.songtracknumber,";
        if (booleanForKey("Year"))
            downloadMetadata += "daap.songyear,";
            
        // chop off the comma
        downloadMetadata = downloadMetadata.substring(0, downloadMetadata.length()-1);
    	
    	return downloadMetadata;
    }
    
    public void run() {
        connect();
    }

    public String getHostname() {
        return this.hostname;
    }
    private void connect() {
        if (hostname.equals(""))
            return;
            
        try {
            byte[] data;
            getAll = false;
            //
            // Request /server-info
            //
//            statusMessage = LocalizedStrings.localizedString("ContactingHost", "ContactingHost") + " " +
//                            hostname;
//			StatusComponent.getInstance().setText(statusMessage);
            network = new NetworkTransferThread(hostname, password, "/server-info");
            network.start();
            while (network.isCompleted() == false) {
                Thread.sleep((int)(Math.random()*100));
            }
            httpStatus = network.getHTTPStatus();
            if (network.didAbort()) {
                errorCode = network.getErrorCode();
                completed = true;
                errorOccured = true;
                return;
            }
            data = network.getBuffer();
            serverInfoResponse = new DaapServerInfoResponse(data);
            String shareName = serverInfoResponse.getItemName();

            //
            // Request /login
            //
//            statusMessage = LocalizedStrings.localizedString("LoggingIn", "LoggingIn") + " " +
//                            shareName;
//            StatusComponent.getInstance().setText(statusMessage);
            network = new NetworkTransferThread(hostname, password, "/login");
            network.start();
            while (network.isCompleted() == false) {
                Thread.sleep((int)(Math.random()*100));
            }
            httpStatus = network.getHTTPStatus();
            if (network.didAbort()) {
                errorCode = network.getErrorCode();
                completed = true;
                errorOccured = true;
                return;
            }

            httpStatus = network.getHTTPStatus();
            if (httpStatus == HttpURLConnection.HTTP_UNAUTHORIZED) {
                // something about the password....
                completed = true;
                errorOccured = true;
                return;
            } else if (httpStatus == HttpURLConnection.HTTP_UNAVAILABLE) {
//                statusMessage = shareName + " " + LocalizedStrings.localizedString("ServerUnavailable","ServerUnavailable");
//                StatusComponent.getInstance().setText(statusMessage);
            	completed = true;
                errorOccured = true;
                return;
            }
            data = network.getBuffer();
            loginResponse = new DaapLoginResponse(data);
            int sessionID = loginResponse.getSessionID();
            
            //
            // Get /update?session-id=sessionID
            //
//            statusMessage = LocalizedStrings.localizedString("UpdatingSession", "UpdatingSession") + " " +
//                            shareName;
//            StatusComponent.getInstance().setText(statusMessage);
            network = new NetworkTransferThread(hostname, password, "/update?session-id="+
                                                sessionID);
            network.start();
            while (network.isCompleted() == false) {
                Thread.sleep((int)(Math.random()*100));
            }
            httpStatus = network.getHTTPStatus();
            if (network.didAbort() == true) {
                errorCode = network.getErrorCode();
                completed = true;
                errorOccured = true;
                return;
            }
            data = network.getBuffer();
            updateResponse = new DaapUpdateResponse(data);
            int revisionNumber = updateResponse.getServerRevision();
            
            //
            // Get /databases?session-id=sessionID&revision-number=revisionNumber
            //
            network = new NetworkTransferThread(hostname, password, "/databases?session-"+
                                                "id="+sessionID+"&revision-number="+
                                                revisionNumber);
            network.start();
            while (network.isCompleted() == false) {
                Thread.sleep((int)(Math.random()*100));
            }
            httpStatus = network.getHTTPStatus();
            if (network.didAbort()) {
                errorCode = network.getErrorCode();
                completed = true;
                errorOccured = true;
                return;
            }
            data = network.getBuffer();
            databasesResponse = new DaapDatabasesResponse(data);
            int databaseID = databasesResponse.getItemID();
            
            //
            // Get the list of playlists
            //
//            statusMessage = LocalizedStrings.localizedString("DownloadingPlaylists", "DownloadingPlaylists") +
//                            " " + shareName;
//            StatusComponent.getInstance().setText(statusMessage);
            network = new NetworkTransferThread(hostname, password, "/databases/"+
                                                databaseID + "/containers?meta=dmap.itemid,"+
                                                "dmap.itemname,dmap.persistentid,"+
                                                "com.apple.itunes.smart-playlist"+
                                                "&session-id="+sessionID+"&revision-number="+
                                                revisionNumber);
            network.start();
            while (network.isCompleted() == false) {
                Thread.sleep((int)(Math.random()*100));
            }
            httpStatus = network.getHTTPStatus();
            if (network.didAbort()) {
                errorCode = network.getErrorCode();
                completed = true;
                errorOccured = true;
                return;
            }
            data = network.getBuffer();
            playlists = new DaapPlaylists(data, parent);
            
            //
            // Get the main playlist
            //
//            statusMessage = LocalizedStrings.localizedString("DownloadingMasterPlaylist",
//                            "DownloadingMasterPlaylist") + " " + shareName;
            this.getAll = true;
//            StatusComponent.getInstance().setText(statusMessage);
            network = new NetworkTransferThread(hostname, password, "/databases/" + databaseID +
                      "/items?type=music&meta=" + this.getDownloadMetdata() + "&session-id=" + sessionID +
                      "&revision-number=" + revisionNumber);
            this.getAll = false;
            network.start();
            while (network.isCompleted() == false) {
                Thread.sleep((int)(Math.random()*100));
            }
            httpStatus = network.getHTTPStatus();
            if (network.didAbort()) {
                errorCode = network.getErrorCode();
                completed = true;
                errorOccured = true;
                return;
            }
            data = network.getBuffer();
//            statusMessage = LocalizedStrings.localizedString("ParsingPlaylist", "ParsingPlaylist") + " " +
//                            shareName;
//            StatusComponent.getInstance().setText(statusMessage);
            songs = new DaapSongs(data, this);
            statusMessage = "";
//            StatusComponent.getInstance().setText(statusMessage);
            completed = true;
        } catch (Exception e) {
            e.printStackTrace();
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            PrintStream ps = new PrintStream(out);
            e.printStackTrace(ps);
            statusMessage = ps.toString();
            completed = true;
            errorOccured = true;
        }
    }
    
    public void logoff() {
        if (loginResponse == null)
            return;
        int sessionID = loginResponse.getSessionID();
        
        network = new NetworkTransferThread(hostname, password, "/logout?session-id=" + sessionID);
        network.start();
        while (network.isCompleted() == false) { }
    }
    
    public void abort() {
        network.abort();
    }


    public NetworkTransferThread getSongThread(DaapSong song) {
        String songURL = "/databases/" + getDatabasesResponse().getItemID()
            + "/items/" + song.getItemID() +"." + song.getFormat()
            + "?session-id=" + getLoginResponse().getSessionID();
        NetworkTransferThread network = new NetworkTransferThread(this.hostname,this.password,songURL);
        return network;
    }





    public DaapSongs getPlaylistSongs(int selectedPlaylist) {
    	String playlistURL = null;
//		StatusComponent.getInstance().setText(LocalizedStrings.localizedString("DownloadingPlaylist", "Downloading Playlist"));
        if(selectedPlaylist == 0) {
            return this.getSongs();
			//playlistURL = "/databases/" + this.getDatabasesResponse().getItemID() +
			//"/items?type=music&meta=" + this.getDownloadMetdata() + "&session-id=" + this.getLoginResponse().getSessionID() +
			//"&revision-number=" + this.updateResponse.getServerRevision();
        } else {
			DaapPlaylist playlist = this.getPlaylists().getPlaylistAtIndex(selectedPlaylist);
		//	playlistURL = "/databases/" +  this.getDatabasesResponse().getItemID() +
		//						 "/containers/" + playlist.getItemID() + "/items?type=music&meta=" +
		//						 this.getDownloadMetdata() + "&session-id=" +
		//						 this.getLoginResponse().getSessionID();
			playlistURL = "/databases/" +  this.getDatabasesResponse().getItemID() +
								 "/containers/" + playlist.getItemID() + "/items?type=music&meta=" +
								 "dmap.itemid" + "&session-id=" +
								 this.getLoginResponse().getSessionID();
        }
        NetworkTransferThread network = new NetworkTransferThread(hostname, password,
                                                                      playlistURL);
        network.start();
        while (network.isCompleted() == false) {
			try {
			    Thread.sleep((int)(Math.random()*100));
			}
			catch ( Exception e ) {
				e.printStackTrace();
			}
        }
        if (network.didAbort()) {
            //String exceptionCaught = NSBundle.localizedString("ExceptionCaught", "ExceptionCaught");
            //ourController.logError("ServerWindowController.downloadPlaylist() " + exceptionCaught +
            //                       ":\n" + network.getException());
        } else {
            byte[] data = network.getBuffer();
            return new DaapSongs(data, this);
        //    songsTableDataSource.setSongs(songs.getSongs());
        }
        return null;
    }

    public DaapServerInfoResponse getServerInfoResponse() {
        return(serverInfoResponse);
    }
    
    public DaapLoginResponse getLoginResponse() {
        return(loginResponse);
    }
    
    public DaapUpdateResponse getUpdateResponse() {
        return(updateResponse);
    }
    
    public DaapDatabasesResponse getDatabasesResponse() {
        return(databasesResponse);
    }
    
    public DaapPlaylists getPlaylists() {
        return(playlists);
    }
    
    public DaapSongs getSongs() {
        return(songs);
    }

    public int getErrorCode() {
        return(errorCode);
    }
    
    public String getNetworkException() {
        return(network.getException());
    }
    
    public String getStatusMessage() {
        return(statusMessage);
    }
    
    public boolean isCompleted() {
        return(completed);
    }
    
    public boolean didErrorOccur() {
        return(errorOccured);
    }
    
    public int getHttpStatus() {
        return(httpStatus);
    }
    
    public String getDownloadMetdata() {
        return getDownloadMeta();
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("\nDaapServer");
        buf.append("\nhostname=" + hostname);
        buf.append("\npassword=" + password);
        buf.append("\nstatusMessage=" + statusMessage);
        buf.append("\ncompleted=" + completed);
        buf.append("\nerrorOccured=" + errorOccured);
        buf.append("\nserverInfoResponse=" + serverInfoResponse);
        buf.append("\nloginResponse=" + loginResponse);
        buf.append("\nupdateResponse=" + updateResponse);
        buf.append("\ndatabasesResponse=" + databasesResponse);
        buf.append("\nplaylists=" + playlists);
        buf.append("\nsongs=" + songs);
        buf.append("\ndownloadMetadata=" + this.getDownloadMetdata());
        return(buf.toString());
    }
    

    
	public boolean booleanForKey(String key){
		if(this.getAll){
			return true;
		}
//		int index = ViewOptionsDialog.colNames3.indexOf(key);
//		if(index!=-1){
//			return getOptions(parent.getViewOptions(), index);
//		}else{
//			System.out.println("key not found: "+key);
//		}
		return false;
	}
    
	private boolean getOptions(int options, int index){
		int temp = (int)(options/Math.pow(2,index))%2;
		return temp==1;
	}
	
	public DaapRemoteSong getSongById(int id) {
		return songs.getSongById(id);
	}
}
