package daapLib;

import java.util.ArrayList;
import java.util.Hashtable;

import daapLib.api.DaapRemoteSong;
import daapLib.api.DaapRemoteSongs;

public class DaapSongs implements DaapRemoteSongs{
    private int databasesongs = -1;
    private int status = -1;
    private int updatetype = -1;
    private int specifiedtotalcount = -1;
    private int returnedcount = 0;
    private int listing = -1;
    private ArrayList songs;
    private DaapServer server;
    private Hashtable table;
	



    public DaapSongs(byte[] data, DaapServer server) {
    	this.table = new Hashtable();
        this.server = server;
        int startOfSong = 0;
        while (startOfSong < data.length) {
            if (data[startOfSong] == 'm' && data[startOfSong+1] == 'l' &&
                data[startOfSong+2] == 'i' && data[startOfSong+3] == 't') {
                
                break;
            } else {
                startOfSong++;
            }
        }
        
        byte[] headerData = new byte[startOfSong];
        System.arraycopy(data, 0, headerData, 0, startOfSong);
        DaapParser parser = new DaapParser();
        parser.parse(headerData);
        
        if (parser.getObjectForKey("adbs") != null)
            databasesongs = ((Integer)parser.getObjectForKey("adbs")).intValue();
        if (parser.getObjectForKey("mstt") != null)
            status = ((Integer)parser.getObjectForKey("mstt")).intValue();
        if (parser.getObjectForKey("muty") != null)
            updatetype = ((Integer)parser.getObjectForKey("muty")).intValue();
        if (parser.getObjectForKey("mtco") != null)
            specifiedtotalcount = ((Integer)parser.getObjectForKey("mtco")).intValue();
        if (parser.getObjectForKey("mrco") != null)
            returnedcount = ((Integer)parser.getObjectForKey("mrco")).intValue();
        if (parser.getObjectForKey("mlcl") != null)
            listing = ((Integer)parser.getObjectForKey("mlcl")).intValue();

        returnedcount=Math.max(returnedcount,0);
        songs = new ArrayList(returnedcount);
        
        int endOfSong = startOfSong + 1;
        while (endOfSong < data.length) {
            if (data[endOfSong] == 'm' && data[endOfSong+1] == 'l' &&
                data[endOfSong+2] == 'i' && data[endOfSong+3] == 't') {
                
                int payloadLength = endOfSong - startOfSong;
                byte[] songData = new byte[payloadLength];
                System.arraycopy(data, startOfSong, songData, 0, payloadLength);
                DaapSong song = new DaapSong(songData, server);
                songs.add(song);
				table.put(new Integer(song.getItemID()), song);
                startOfSong = endOfSong;
                endOfSong += 4;
            }
            endOfSong++;
        }
        // And once more, for the final item
        int payloadLength = data.length - startOfSong;
        byte[] songData = new byte[payloadLength];
        System.arraycopy(data, startOfSong, songData, 0, payloadLength);
        DaapSong song = new DaapSong(songData, server);
        songs.add(song);
        table.put(new Integer(song.getItemID()), song);
        if (songs.size() != returnedcount) {
            // Find a better way to handle this error
        }
    }
    
    public int getDatabaseSongs() {
        return(databasesongs);
    }
    
    public int getStatus() {
        return(status);
    }
    
    public int getUpdateType() {
        return(updatetype);
    }
    
    public int getSpecifiedTotalCount() {
        return(specifiedtotalcount);
    }
    
    public int getReturnedCount() {
        return(returnedcount);
    }
    
    public int getListing() {
        return(listing);
    }
    
    public int count() {
        return(songs.size());
    }
    
    public DaapRemoteSong getSongAtIndex(int index) {
        return((DaapRemoteSong)songs.get(index));
    }
    
    public ArrayList getSongList() {
        return(songs);
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("\nSongs");
        buf.append("\ndaap.databasesongs=" + databasesongs);
        buf.append("\ndmap.status=" + status);
        buf.append("\ndmap.updatetype=" + updatetype);
        buf.append("\ndmap.specifiedtotalcount=" + specifiedtotalcount);
        buf.append("\ndmap.returnedcount=" + returnedcount);
        buf.append("\ndmap.listing=" + listing);
        buf.append("\nsongs.size()=" + songs.size());
        return(buf.toString());
    }
    // Method used by the Master-Playlist
    public DaapRemoteSong getSongById(int id){
    	return (DaapRemoteSong)table.get(new Integer(id));
    }

	/* (non-Javadoc)
	 * @see ch.toenz.jTunes.Songs#getSongCount()
	 */
	public int getSongCount() {
		return songs.size();
	}

}
