package daapLib.network;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.net.ConnectException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.HashMap;

public class NetworkTransferThread extends Thread {
	public static final int SERVER_DOWN = 1;
	public static final int SERVER_NOT_LISTENING = 2;
	public static final int UNABLE_TO_RESOLVE = 3;
	public static final int USER_ABORTED = 4;
	public static final int UNEXPECTED_FAILURE = 5;
	public static final int EXCEPTION_THROWN = 6;
	public static final int UNSUPPORTED_SERVER_VERSION = 7;

	public static int PORT = 3689;
	public static String HTTP_VERS = "HTTP/1.0";

	BufferedInputStream in;
	PrintWriter out;
	String host;
	String theURL;
	String password;
	String downloadFilePath;
	int totalread;
	int contentLength;
	int httpStatus;
	boolean completed;
	boolean abort;
	byte[] buffer;
	String exception;
	int errorCode;

	public NetworkTransferThread(String host, String password, String theURL) {
		this(host, password, theURL, null);
	}

	public NetworkTransferThread(
		String host,
		String password,
		String theURL,
		String downloadFilePath) {
		this.host = host;
		if (password != null)
			this.password = password;
		else
			this.password = "";
		this.theURL = theURL;
		this.downloadFilePath = downloadFilePath;
		errorCode = -1;
		totalread = 0;
		contentLength = 0;
		httpStatus = 0;
		completed = false;
		abort = false;
		buffer = null;
		exception = null;
	}

	public void run() {
		boolean useGzip = false;

		if (theURL == null || host == null) {
			abort();
			return;
		}

		try {
			Socket s = new Socket(host, PORT);
			in = new BufferedInputStream(s.getInputStream());
			out = new PrintWriter(s.getOutputStream());

			out.print(sendURL(theURL, password));

			out.flush();

			while (in != null) {
				if (in.available() > 0) {
					HashMap httpHeaders = getHTTPHeaders();
					httpStatus =
						Integer
							.valueOf((String) httpHeaders.get("HTTP Status"))
							.intValue();
					contentLength =
						Integer
							.valueOf((String) httpHeaders.get("Content-Length"))
							.intValue();
							//TOTHINK ̰Ͷ   ȵǴ  ƴұ?
					if (httpHeaders
						.get("DAAP-Server")
						.equals("iTunes/4.0 (Mac OS X)")
						== false) {
						errorCode = UNSUPPORTED_SERVER_VERSION;
						abort();
						in.close();
						out.close();
						return;
					}
					break;
				}
			}

			FileOutputStream outfile = null;
			if (downloadFilePath != null) {
				outfile = new FileOutputStream(downloadFilePath, false);
			} else {
				buffer = new byte[contentLength];
			}

			while (totalread < contentLength && abort == false) {
				int available = in.available();
				int curPos = 0;
				if (available > 0) {
					byte[] tmpbuffer = new byte[available];
					int readsize = in.read(tmpbuffer, 0, available);
					if (outfile != null) {
						outfile.write(tmpbuffer);
					} else {
						for (int i = 0; i < readsize; i++) {
							buffer[totalread + i] = tmpbuffer[i];
						}
					}
					totalread += readsize;
				}
				Thread.sleep((int) (Math.random() * 100));
			}
			if (outfile != null) {
				outfile.close();
			}
			out.close();
			in.close();

			completed = true;
			if (abort == true) {
				buffer = null;
				if (downloadFilePath != null) {
					File outputFile = new File(downloadFilePath);
					outputFile.delete();
				}
			}
		} catch (ConnectException e) {
			String exceptionMessage = e.getMessage();
			if (exceptionMessage.equals("Operation timed out")) {
				errorCode = SERVER_DOWN;
			} else if (exceptionMessage.equals("Connection refused")) {
				errorCode = SERVER_NOT_LISTENING;
			} else {
				exception = getExceptionStackTrace(e);
			}
			abort();
			return;
		} catch (UnknownHostException e) {
			errorCode = UNABLE_TO_RESOLVE;
			abort();
			return;
		} catch (Exception e) {
			errorCode = EXCEPTION_THROWN;
			exception = getExceptionStackTrace(e);
			abort();
			return;
		}
	}

	public String readLine() {
		String line = new String();
		try {
			while (in.available() > 0) {
				int content = in.read();
				if (content == 0x0d) { // '\r'
					content = in.read();
					if (content == 0x0a) { // '\n', end of line.
						break;
					} else {
						line += (char) 0x0d + "" + (char) content;
					}
				} else {
					line += (char) content;
				}
			}
		} catch (Exception e) {
			exception = getExceptionStackTrace(e);
			abort();
		}
		return line;
	}

	public HashMap getHTTPHeaders() {
		HashMap headers = new HashMap();
		try {
			while (in.available() > 0) {
				String httpHeader = readLine();
				if (httpHeader == null) {
					return (headers);
				} else if (httpHeader.equals("")) {
					return (headers);
				}

				String key = new String();
				String value = new String();
				if (httpHeader.startsWith("HTTP/1")) {
					int delimiter = httpHeader.indexOf(" ");
					httpHeader = httpHeader.substring(delimiter + 1);
					delimiter = httpHeader.indexOf(" ");
					key = "HTTP Status";
					value = httpHeader.substring(0, delimiter);
				} else {
					int delimiter = httpHeader.indexOf(":");
					key = httpHeader.substring(0, delimiter);
					value = httpHeader.substring(delimiter + 2);
				}
				headers.put(key, value);
			}
		} catch (Exception e) {
			exception = getExceptionStackTrace(e);
			abort();
		}
		return (headers);
	}

	public int getHTTPStatus() {
		return (httpStatus);
	}

	public int getTotalRead() {
		return (totalread);
	}

	public int getContentLength() {
		return (contentLength);
	}

	public boolean isCompleted() {
		return (completed);
	}

	public byte[] getBuffer() {
		return (buffer);
	}

	public void abortAction() {
		errorCode = USER_ABORTED;
		abort();
	}

	public void abort() {
		abort = true;
		completed = true;
	}

	public boolean didAbort() {
		return (abort);
	}

	public String getException() {
		return (exception);
	}

	public int getErrorCode() {
		return (errorCode);
	}

	public String getExceptionStackTrace(Exception e) {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		PrintStream ps = new PrintStream(out);
		e.printStackTrace(ps);
		return (out.toString());
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("\nNetworkTransferThread");
		buf.append("\nin=" + in);
		buf.append("\nout=" + out);
		buf.append("\nhost=" + host);
		buf.append("\npassword=" + password);
		buf.append("\ntheURL=" + theURL);
		buf.append("\ndownloadFilePath=" + downloadFilePath);
		buf.append("\ntotalread=" + totalread);
		buf.append("\ncontentLength=" + contentLength);
		buf.append("\nhttpStatus=" + httpStatus);
		buf.append("\ncompleted=" + completed);
		buf.append("\nabort=" + abort);
		buf.append("\nbuffer=" + buffer);
		if (buffer != null)
			buf.append("\nbuffer.length=" + buffer.length);
		buf.append("\nexception=" + exception);
		return (buf.toString());
	}

	public String sendURL(String theURL, String password) {
		//DAAP-Client-Version: 1.0
		StringBuffer bu = new StringBuffer("");
		bu.append("GET " + theURL + " " + HTTP_VERS + "\r\n");
		if (!password.equals("")) {
			Base64 base64 = new Base64();
			String encodedPassword = base64.encode("iTunes_4.0:" + password);
			bu.append("Authorization: Basic " + encodedPassword + "\r\n");
		}
		bu.append("\r\n");
		return bu.toString();
	}

	public BufferedInputStream getStream() {
		try {
			Socket s = new Socket(host, PORT);
			in = new BufferedInputStream(s.getInputStream());
			out = new PrintWriter(s.getOutputStream());
			out.print(sendURL(theURL, password));
			out.flush();
			HashMap httpHeaders = this.getHTTPHeaders();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return in;
	}

}
