package laf.mac;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JViewport;
import javax.swing.SwingConstants;
import javax.swing.border.Border;

import laf.mac.renderedUI.ComponentRenderer;
import laf.mac.renderedUI.RenderedPanel;
import laf.mac.renderedUI.RenderedProgressBarUI;
import laf.mac.renderedUI.RenderedScrollBarUI;
import laf.mac.renderedUI.RenderedSliderUI;
import laf.mac.renderedUI.RenderedTableCell;
import laf.mac.renderedUI.RenderedTableHeader;
import laf.mac.renderedUI.RenderedTextUI;
import laf.mac.texturedUI.ComponentTexturer;
import laf.mac.texturedUI.TexturedButtonBorder;
import laf.mac.texturedUI.TexturedCheckBoxMenuItemUI;
import laf.mac.texturedUI.TexturedLabelUI;
import laf.mac.texturedUI.TexturedMenuBarUI;
import laf.mac.texturedUI.TexturedMenuItemUI;
import laf.mac.texturedUI.TexturedMenuUI;
import laf.mac.texturedUI.TexturedPanelUI;
import laf.mac.texturedUI.TexturedRadioButtonMenuItemUI;
import laf.mac.texturedUI.TexturedSplitPaneUI;
import laf.mac.texturedUI.TexturedWindowBorder;

/*
 * Created on 2004. 1. 14.
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

/**
 * @author Administrator
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public final class MacLafFactory {
	private static ComponentTexturer texturer = new MetalTexturer();
	private static Border windowBorder = new TexturedWindowBorder();
	private static Border buttonBorder = new TexturedButtonBorder(texturer);
	private static ComponentRenderer renderer = new AquaRenderer();

	/**
	 * @return
	 */
	public static JFrame createFrame() {
		MacFrame macFrame = new MacFrame();
		macFrame.setUI(new TexturedPanelUI(texturer));		
		macFrame.setBorder(windowBorder);
		return macFrame;
	}

	/**
	 * @return
	 */
	public static JMenuBar createMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		menuBar.setUI(new TexturedMenuBarUI(texturer));
		menuBar.setOpaque(false);
		return menuBar;
	}

	/**
	 * @return
	 */
	public static JMenu createMenu() {
		JMenu menu = new JMenu(){};
		menu.setUI(new TexturedMenuUI(texturer));
		menu.setOpaque(false);
		return menu;
	}

	/**
	 * @return
	 */
	public static JMenuItem createMenuItem() {
		JMenuItem item = new JMenuItem();
		item.setUI(new TexturedMenuItemUI(texturer));
		item.setOpaque(false);
		return item;
	}

	/**
	 * @return
	 */
	public static JCheckBoxMenuItem createCheckBoxMenuItem() {
		JCheckBoxMenuItem check = new JCheckBoxMenuItem();
		check.setUI(new TexturedCheckBoxMenuItemUI(texturer));
		check.setOpaque(false);
		return check;
	}

	/**
	 * @return
	 */
	public static JRadioButtonMenuItem createRadioButtonMenuItem() {
		JRadioButtonMenuItem radio = new JRadioButtonMenuItem();
		radio.setUI(new TexturedRadioButtonMenuItemUI(texturer));
		radio.setOpaque(false);
		return radio;
	}

	/**
	 * @return
	 */
	public static JSplitPane createSplitPane() {
		JSplitPane splite = new JSplitPane();
		splite.setUI(new TexturedSplitPaneUI(texturer));
		splite.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
		splite.setOpaque(false);
		return splite;
	}

	/**
	 * @return
	 */
	public static JScrollPane createScrollPane() {
		JScrollPane scroll = new JScrollPane();
		scroll.getHorizontalScrollBar().setUI(new RenderedScrollBarUI(renderer,SwingConstants.HORIZONTAL));
		scroll.getVerticalScrollBar().setUI(new RenderedScrollBarUI(renderer,SwingConstants.VERTICAL));
		scroll.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
//		TOTHINK: հ ߿ ؾ 		
//		scroll.getViewport().setBackground(Color.WHITE);		
		scroll.setAutoscrolls(true);
		return scroll;
	}

	/**
	 * @return
	 */
	public static JPanel createPanel() {
		JPanel panel = new JPanel(){
			/* (non-Javadoc)
			 * @see java.awt.Container#addImpl(java.awt.Component, java.lang.Object, int)
			 */
			protected void addImpl(
				Component comp,
				Object constraints,
				int index) {
				if(comp instanceof JPanel){
					((JPanel)comp).setUI(new TexturedPanelUI(texturer));
					
				}
				super.addImpl(comp, constraints, index);
			}
		};
//		JPanel panel = new JPanel();
		panel.setUI(new TexturedPanelUI(texturer));
		panel.setOpaque(false);
		return panel;
	}

	/**
	 * @return
	 */
	public static JLabel createLabel() {
		JLabel label = new JLabel();
		label.setHorizontalAlignment(JLabel.CENTER);
		label.setUI(new TexturedLabelUI(texturer));
		label.setOpaque(false);
		return label;
	}

	/**
	 * @return
	 */
	public static JTable createTable() {
		final JTable table = new JTable();
		table.getTableHeader().setDefaultRenderer(new RenderedTableHeader(renderer));
		table.setDefaultRenderer(Object.class,new RenderedTableCell(renderer));
//		TOTHINK: հ ߿ ؾ 
//		table.setUI()
		table.setShowHorizontalLines(false);
		table.getTableHeader().setBackground(Color.WHITE);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		return table;
	}

	/**
	 * @return
	 */
	public static JSlider createSlider() {
		JSlider slider = new JSlider();
		slider.setUI(new RenderedSliderUI(slider,renderer));
		slider.setFocusable(false);
		slider.setOpaque(false);
		return slider;
	}

	/**
	 * @param string
	 * @return
	 */
	public static JLabel createImageLabel(String string) {	
		return new JLabel((Icon)renderer.getLabelTable().get(string));
	}

	/**
	 * @param string
	 * @return
	 */
	public static JButton createImageButton(String string) {
		JButton button = new JButton();
		button.setIcon((Icon)renderer.getButtonTable().get(string+"Button"));
		button.setPressedIcon((Icon)renderer.getButtonTable().get(string+"ButtonPress"));
		button.setDisabledIcon((Icon)renderer.getButtonTable().get(string+"ButtonDisable"));
		button.setOpaque(false);
		button.setMargin(new java.awt.Insets(0,0,0,0));
		button.setBorder(javax.swing.BorderFactory.createEmptyBorder(0,2,2,2));
		button.setFocusPainted(false);
		button.setFocusable(false);
		button.setPreferredSize(new Dimension(button.getIcon().getIconWidth(),button.getIcon().getIconHeight()));
		return button;
	}

	/**
	 * @param string
	 * @return
	 */
	public static JToggleButton createImageToggleButton(String string) {
		// TODO repeat 1 ϴ  ߰
		if(!string.equals("Repeat")){
			JToggleButton toggle = new JToggleButton();
			toggle.setIcon((Icon)texturer.getButtonTable().get(string+"Button"));
			toggle.setSelectedIcon((Icon)texturer.getButtonTable().get(string+"ButtonSelect"));
			toggle.setBorder(buttonBorder);
			toggle.setMargin(new java.awt.Insets(0,0,0,0));
			toggle.setOpaque(false);
			toggle.setFocusPainted(false);
			return toggle;
		}
		return null;
	}

	/**
	 * @return
	 */
	public static JTextField createTextField() {
		JTextField text = new JTextField();
		text.setUI(new RenderedTextUI(text,renderer));
		text.setBorder(BorderFactory.createEmptyBorder());
		text.setSelectionColor(renderer.getColor("selection"));
		text.setSelectedTextColor(renderer.getColor("selectionForground"));
		return text;
	}

	/**
	 * @return
	 */
	public static JProgressBar createProgressBar() {
		JProgressBar progress = new JProgressBar();
		progress.setUI(new RenderedProgressBarUI(renderer));
		return progress;
	}

	/**
	 * @return
	 */
	public static JPanel createDisplayPanel() {
		JPanel panel = new JPanel();
		panel.setUI(new RenderedPanel(renderer));
		return panel;
	}

	/**
	 * @return
	 */
	public static JList createList() {
		JList list = new JList();
		list.setSelectionForeground(renderer.getColor("selectionForground"));
		list.setSelectionBackground(renderer.getColor("selection"));
		return list;
	}

}
