package model.impl;
import java.io.IOException;
import java.util.ArrayList;

import javax.jmdns.JmDNS;
import javax.jmdns.ServiceInfo;
import javax.jmdns.ServiceListener;

import model.api.Library;
import model.api.LibraryChangeListener;
import model.impl.local.LocalFactory;
import model.impl.remote.RemoteFactory;


public class LibraryManager implements ServiceListener {


	private JmDNS _dns;
	public final static int LOCAL_LIBRARY_INDEX = 0;	
	public final static String LOCAL_LIB_FILE_NAME = "iTunes Music Library.xml";
	public String localLibLocation ="C:/Documents and Settings/Administrator/My Documents/My Music/iTunes/";
	private ArrayList _listeners;
	ArrayList _libraries;

	public LibraryManager(String string) {
		localLibLocation = string;
		_listeners = new ArrayList();
		_libraries = new ArrayList();
		init();
	}

	public void close() {
		if (_dns != null) {
			_dns.close();
			_dns = null;
		}
	}

	void init() {
		initLocalLibrary();
		initRemotemanager();
	}

	/**
	 * 
	 */
	private void initRemotemanager() {
		// zeroconf ʱȭ
		initZeroconfHandler();

	}

	/**
	 * 
	 */
	private void initZeroconfHandler() {
		try {
			_dns = new JmDNS();
			//TODO DAAP Service Type Ȯ !!!
			_dns.addServiceListener("_daap._tcp.local.", this);

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	private void initLocalLibrary() {
		//TODO XML ġ ĥ !!
		_libraries.add(
			LOCAL_LIBRARY_INDEX,
			LocalFactory.getLibrary(localLibLocation+java.io.File.separator+LOCAL_LIB_FILE_NAME));

	}

	public Library[] getLibraries() {
		return (Library[]) _libraries.toArray(new Library[0]);
	}

	public Library getLibrary(int index) {
		return (Library) _libraries.get(index);
	}

	public int size() {
		return _libraries.size();
	}

	public void addListener(LibraryChangeListener listener) {
		_listeners.add(listener);
	}

	public void removeListener(LibraryChangeListener listener) {
		_listeners.remove(listener);
	}

	void notifyLibraryRegistered(Library lib) {
		for (int i = 0; i < _listeners.size(); i++) {
			LibraryChangeListener listener =
				(LibraryChangeListener) _listeners.get(i);
			listener.libraryRegistered(lib);
		}
	}

	void notifyLibraryUnregistered(Library lib) {
		for (int i = 0; i < _listeners.size(); i++) {
			LibraryChangeListener listener =
				(LibraryChangeListener) _listeners.get(i);
			listener.libraryUnregistered(lib);
		}
	}

	/**
	 * @param jmdns
	 * @param type
	 * @param name
	 * @return
	 */
	private Library connectRemoteServer(JmDNS jmdns,String type,String name) {
		
		String hostname = jmdns.getServiceInfo(type,name).getAddress();
		
		return new RemoteFactory(hostname,"").getLibrary();
	}

	/**
	 * @param jmdns
	 * @param type
	 * @param name
	 * @return
	 */
	private Library disconnectRemoteServer(JmDNS jmdns,String type,String name) {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see javax.jmdns.ServiceListener#addService(javax.jmdns.JmDNS, java.lang.String, java.lang.String)
	 */
	public void addService(JmDNS jmdns, String type, String name) {
		// TODO Auto-generated method stub
		Library lib = connectRemoteServer(jmdns, type, name);
		if (lib != null) {
			_libraries.add(lib);
			notifyLibraryRegistered(lib);
		}
	}

	/* (non-Javadoc)
	 * @see javax.jmdns.ServiceListener#removeService(javax.jmdns.JmDNS, java.lang.String, java.lang.String)
	 */
	public void removeService(JmDNS jmdns, String type, String name) {
		// TODO Auto-generated method stub
		Library lib = disconnectRemoteServer(jmdns, type, name);
		if (lib != null) {
			_libraries.remove(lib);
			notifyLibraryUnregistered(lib);
		}
	}

	/* (non-Javadoc)
	 * @see javax.jmdns.ServiceListener#resolveService(javax.jmdns.JmDNS, java.lang.String, java.lang.String, javax.jmdns.ServiceInfo)
	 */
	public void resolveService(
		JmDNS jmdns,
		String type,
		String name,
		ServiceInfo info) {
		// TODO Auto-generated method stub
		//  ޼ҵ ص   
	}
}
