package model.impl.local;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import model.api.Library;
import model.api.Playlist;
import model.api.Song;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author Jeong Ingu
 */
public class LocalXMLLibrary implements Library {
	public static String _musicFolder;

	String _owner;
	Document _doc;
	Element _rootElement;

	Hashtable _songTable;
	Vector _songs;
	Vector _playlist;
	Node _trackNode;
	Node _playListNode;

	public static void setMusicFloder(String musicFolder) {
		_musicFolder = musicFolder;
		// TODO   ٲ 쿣  α׷  ؼ ʱȭ ش

	}

	public LocalXMLLibrary(String xmlLocation) {
		this(new File(xmlLocation));

		// TODO XML  ֱ
		//xmlLocation = "./iTunes Music Library Proto.xml";
	}

	public LocalXMLLibrary(File xmlFile) {

		try {
			_doc = xml2dom(xmlFile);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		}

		_rootElement = _doc.getDocumentElement();

		initSongs(_doc);
		Hashtable songIds = initSongIds(_doc);
		initPlaylists(_doc,songIds);
		setSongIdInTrackId();
	}


	private void setSongIdInTrackId() {
		NodeList trackList = getTrackList();
		int length = trackList.getLength(); 
		for(int i=0;i<length-1 ; i+=4) {
			String newId = trackList.item(i + 3).getChildNodes().item(0).getNodeValue();
			trackList.item(i + 1).getChildNodes().item(0).setNodeValue(""+newId);
							
		}
	}

	private Hashtable initSongIds(Document _doc) {
		Hashtable songIds = new Hashtable();

		//_trackNode = getTrackNode();

		NodeList trackList = getTrackList();

		for (int i = 0; i < trackList.getLength() - 1; i += 4) {
			String prevId = trackList.item(i + 1).getChildNodes().item(0).getNodeValue();
			String newId = trackList.item(i + 3).getChildNodes().item(2).getChildNodes().item(0).getNodeValue();

			songIds.put(prevId,newId);

		}
		return songIds;
	}
	private void initPlaylists(Document doc, Hashtable songIds) {
		_playlist = new Vector();

		_playListNode = getPlayListNode();

		NodeList playLists = getPlayLists();

		//TODO Library ÷ Ʈ 
		for (int i = 1; i < playLists.getLength() - 1; i += 2) {

			Node dictNode = playLists.item(i);
		
			Playlist playlist = createPlayList(dictNode, songIds);
			_playlist.add(playlist);

		}

	}

	/**
	 * @param list
	 * @return
	 */
	private Playlist createPlayList(Node node, Hashtable songIds) {

		return new LocalXMLPlaylist(node, getAllSongs(), songIds);
	}
	
	private Document xml2dom(File xmlFile)
			throws ParserConfigurationException, SAXException {

			DocumentBuilder db = null;
			try {
				DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
				db = dbf.newDocumentBuilder();
				_doc = db.parse(xmlFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
			return _doc;
		}

	private NodeList getPlayLists() {

		return _playListNode.getChildNodes();
	}

	private Node getPlayListNode() {
		return _doc
			.getDocumentElement()
			.getChildNodes()
			.item(1)
			.getChildNodes()
			.item(
			19);

	}
	/**
	 * @param _doc
	 * @return
	 */
	private void initSongs(Document doc) {
		_songs = new Vector();

		_trackNode = getTrackNode();

		NodeList trackList = getTrackList();

		for (int i = 0; i < trackList.getLength() - 1; i += 4) {
			//Node node = trackList.item(i);
			Node dictNode = trackList.item(i + 3);

			Song song = createSong(dictNode);
			_songs.add(song);

		}
	}

	/**
	 * @return
	 */
	private NodeList getTrackList() {
		return _trackNode.getChildNodes();
	}

	/**
	 * @return
	 */
	private Node getTrackNode() {
		return _rootElement.getChildNodes().item(1).getChildNodes().item(15);
	}

	private Song createSong(Node dictNode) {
		return new LocalXMLSong(dictNode);
	}

	public String getOwner() {
		return _owner;
	}

	public Playlist getPlaylist(int index) {
		return (Playlist) _playlist.get(index);
	}

	public Playlist[] getAllPlaylists() {	
		return (Playlist[]) _playlist.toArray(new LocalXMLPlaylist[0]);	
	}

	/* (non-Javadoc)
	 * @see api.MusicStore#addPlaylist(api.Playlist)
	 */
	public Playlist createPlaylist(String playlistName) {

		LocalXMLPlaylist xmlPlaylist =
			new LocalXMLPlaylist(playlistName, getPlayLists());
		_playlist.add(xmlPlaylist);
		addPlayListNode(xmlPlaylist.getXMLNode());
		return (Playlist) xmlPlaylist;
	}
	/**
	 * @param node
	 */
	private void addPlayListNode(Node node) {
		Node playListNode = getPlayListNode();
		playListNode.insertBefore(getTextNode(), playListNode.getLastChild());
		playListNode.insertBefore(node, playListNode.getLastChild());

	}

	/* (non-Javadoc)
	 * @see api.MusicStore#removePlaylist(api.Playlist)
	 */
	public void removePlaylist(Playlist playlist) {

		int index = findIndexInSongsNode(playlist);
		delPlaylistInNode(_playListNode.getChildNodes().item(index));
		delPlaylistInNode(_playListNode.getChildNodes().item(index - 1));
		_playlist.remove(playlist);
	}
	/**
	 * @param node
	 * @return
	 */
	private void delPlaylistInNode(Node node) {
		_playListNode.removeChild(node);
	}

	/**
	 * @param playlist
	 * @return
	 */
	private int findIndexInSongsNode(Playlist playlist) {
		for (int i = 1;
			i < _playListNode.getChildNodes().getLength() - 1;
			i += 2) {
			String playlistName =
				_playListNode
					.getChildNodes()
					.item(i)
					.getChildNodes()
					.item(2)
					.getChildNodes()
					.item(0)
					.getNodeValue();
			System.err.println(playlistName);
			if (playlist.getName().equals(playlistName)) {
				return i;
			}
		}
		System.err.println("Not found playlist");
		return 0;
	}

	/* (non-Javadoc)
	 * @see api.MusicStore#getAllSong()
	 */
	public Song[] getAllSongs() {
		return (Song[]) _songs.toArray(new LocalXMLSong[0]);
	}

	public void addSong(Song song) { // ߰
		
		LocalXMLSong xmlSong = new LocalXMLSong(song, getTrackList());
		_songs.add(xmlSong);
		addTrackNode(xmlSong.getId(), xmlSong.getXMLNode());
		addTrackIdInLibrary(song);

	}
	private void addTrackIdInLibrary(Song song) {
		getPlaylist(1).addSong(song);
		
	}

	private void addTrackNode(int trackId, Node songNode) {
		int trackIdIndex = getTrackList().getLength() + 1;
		Node trackNode = getTrackNode();
		trackNode.insertBefore(getTextNode(), getTrackNode().getLastChild());
		trackNode.insertBefore(
			changeTrackIdNode(trackId, trackIdIndex),
			getTrackNode().getLastChild());
		trackNode.insertBefore(getTextNode(), getTrackNode().getLastChild());
		trackNode.insertBefore(songNode, getTrackNode().getLastChild());

	}

	private Node changeTrackIdNode(int trackId, int trackIdIndex) {

		Node node = getTrackList().item(1).cloneNode(true);
		node.setNodeValue("" + trackId);
		return node;
	}

	private Node getTextNode() {
		return getTrackList().item(0).cloneNode(true);
	}

	public void removeSong(int index) { // 
		Song song = (Song) _songs.get(index);
		_songs.remove(index);
		delSongInTrack(getTrackList(), song);
		delTrackIdInLibrary(index);
	}
	private void delTrackIdInLibrary(int index) {
		getPlaylist(1).removeSong(index);
		
	}

	private void delSongInTrack(NodeList list, Song song) {
		int index = findTrackIndex(song);

	}
	private int findTrackIndex(Song song) {
		for (int i = 0; i < getTrackList().getLength() - 1; i++) {
			String trackId = getTrackList().item(i).getNodeValue();
			if (trackId.equals("" + song.getId())) {
				return i;
			}
		}
		System.err.println("Not Found Song In Track");
		return 0;
	}

	public void save() {
		dom2xml(_doc);
	}

	private void dom2xml(Document doc) {
		try {
			TransformerFactory transFactory = TransformerFactory.newInstance();
			Transformer transformer = transFactory.newTransformer();

			DOMSource source = new DOMSource(doc);

			StreamResult result =
				new StreamResult(
					new FileOutputStream(
						new File("c:/Documents and Settings/Administrator/My Documents/My Music/iTunes/Library.xml")));
			transformer.transform(source, result);
		} catch (Exception e) {
			System.out.println("Exception :" + e);
		}

	}

	public Song getSong(int index) {
		return (LocalXMLSong) _songs.get(index);
	}
	public int getPlaylistSize() {
		return _playlist.size();
	}

	public int getSongSize() {
		return _songs.size();
	}

	public void setName(String name) {
		((Playlist)_playlist.get(0)).setName(name);
		setElement("Name", name);
	}
	public void setPlaylistId(int id) {
		((Playlist)_playlist.get(0)).setPlaylistId(id);
		setElement("Playlist ID", "" + id);		
	}
	public void setPlaylistPersistentId(String persistentId) {
		((Playlist)_playlist.get(0)).setPlaylistPersistentId(persistentId);
		setElement("Playlist Persistent ID", persistentId);
	}
	private void setElement(String string, String nodeValue) {
		NodeList list = _playListNode.getChildNodes();

		int i = 0;
		if ("Name"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			if ("Name".equals(string)) {
				list.item(i + 2).getChildNodes().item(0).setNodeValue(
					"" + nodeValue);
			}
			i += 3;
		}
		if ("Master"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			i += 3;
		}
		if ("Playlist ID"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			if ("Playlist ID".equals(string)) {
				list.item(i + 2).getChildNodes().item(0).setNodeValue(
					"" + nodeValue);
			}
			i += 3;
		}
		if ("Playlist Persistent ID"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			if ("Playlist Persistent ID".equals(string)) {
				list.item(i + 2).getChildNodes().item(0).setNodeValue(
					"" + nodeValue);
			}
			i += 3;
		}

	}
	public String getName() {
		return ((Playlist)_playlist.get(0)).getName();
	}
	public int getPlaylistId() {
		return ((Playlist)_playlist.get(0)).getPlaylistId();
	}

	public String getPlaylistPersistentId() {
		return ((Playlist)_playlist.get(0)).getPlaylistPersistentId();
	}

	/* (non-Javadoc)
	 * @see api.Playlist#isSmartList()
	 */
	public boolean isSmartList() {
		// TODO Auto-generated method stub
		return false;
	}

}
