package model.impl.local;

import java.util.Hashtable;
import java.util.Vector;

import model.api.Playlist;
import model.api.Song;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * @author Jeong Ingu
 * 
 */
public class LocalXMLPlaylist implements Playlist {

	private boolean _haveNotSongs;
	private boolean _isSmartList;
	private String _name;
	private int _playlistId;
	private String _playlistPersistentId;
	private Vector _songs;

	private Node _xmlNode;
	private Node _xmlSongsNode;
	private Node _songTextNode;
	private Node _songElementNode;

	private int _arrayNodeIndex;

	public LocalXMLPlaylist(String playlistName, NodeList playNodelist) {
		this();
		createEmptyNode(playNodelist);
		setName(playlistName);
	}

	private void createEmptyNode(NodeList list) {
		cloneListNode(list);
		formatListNode();
	}
	private void formatListNode() {
		formatPlaylistInfo(getXMLNode());
		formatSongsNode(getXMLSongsNode());

	}

	private void formatSongsNode(Node songsNode) {
		Node songTextNode = songsNode.getChildNodes().item(0).cloneNode(true);
		Node songElementNode =
			songsNode.getChildNodes().item(1).cloneNode(true);

		while (songsNode.getChildNodes().getLength() > 3) {
			songsNode.removeChild(songsNode.getChildNodes().item(0));
		}
		setSongInNode(_xmlSongsNode.getChildNodes().item(1), "0");

	}
	private void formatPlaylistInfo(Node xmlNode) {

		for (int i = 4; i <= 6; i++) {

			Node removeNode = xmlNode.getChildNodes().item(4);
			xmlNode.removeChild(removeNode);
		}
		xmlNode.getChildNodes().item(5).getChildNodes().item(0).setNodeValue(
			"" + null);
		xmlNode.getChildNodes().item(8).getChildNodes().item(0).setNodeValue(
			"" + null);

		_arrayNodeIndex = 15;
		_xmlSongsNode = _xmlNode.getChildNodes().item(_arrayNodeIndex);

	}
	private void cloneListNode(NodeList list) {
		_xmlNode = list.item(1).cloneNode(true);

	}
	private LocalXMLPlaylist() {
		_songs = new Vector();
	}
	public LocalXMLPlaylist(Node playlistNode, Song[] songs , Hashtable songIds) {
		//xmlϷ  ʱȭ
		this();
		_xmlNode = playlistNode;
		formatPlaylistInfo(playlistNode.getChildNodes());
		if(_isSmartList) {
			// TODO smartList will be maked
			System.err.println(this.getName()+" This Playlist is SmartList");
		}else if(_haveNotSongs ) {
			// TODO empty list
		System.err.println(this.getName()+" This Playlist have Not Songs");
		}else {
			initSongs(songs, songIds);
		}
	}
	private void initSongs(Song[] songs, Hashtable songIds) {
		int length = _xmlSongsNode.getChildNodes().getLength();
		
		for (int i=1; i<length-1; i+=2) {
			String songid = _xmlSongsNode.getChildNodes().item(i).getChildNodes().item(2).getChildNodes().item(0).getNodeValue();
			int newId = Integer.parseInt(""+songIds.get(songid));
			_xmlSongsNode.getChildNodes().item(i).getChildNodes().item(2).getChildNodes().item(0).setNodeValue(""+ newId);
			
			Song song =	findSong(newId, songs);
			_songs.add(song);
		}
		
	}

	private Song findSong(int songid,Song[] songs) {
		for(int i=0; i<songs.length ; i++) {
			if(songid == songs[i].getId()) {
				return songs[i];
			}
		}
		System.err.println("Not Found Song In Track");
		return null;
	}

	private void formatPlaylistInfo(NodeList list) {
		int i = 0;
		String keyName = "";
		String value = "";
		endTextNode(list);
		for (i = 0; i < list.getLength() - 1; i++) {
			if (list.item(i) == TextNode(list)) {
				continue;
			} else {
				String valueKeyType = list.item(i).getNodeName();

				if ("key".equals(valueKeyType)) {
					keyName =
						list.item(i).getChildNodes().item(0).getNodeValue();
					if (list.item(i + 1).getNodeName() == "true") {
						continue;
					} else if ("Smart Info".equals(keyName)) {
						_isSmartList = true;
						break;
					} else if ("Playlist Items".equals(keyName)) {
						break;
					}
					if (list.item(i + 1) != TextNode(list)) {
						value =
							list
								.item(i + 1)
								.getChildNodes()
								.item(0)
								.getNodeValue();
						setAttribute(keyName, value);
					}

				}

			}
		}
		if (i + 2 < list.getLength() - 1) {
			_xmlSongsNode = list.item(i + 2);
		}else {
			_haveNotSongs = true; 
		}
	}

	/**
	 * @param list
	 * @return
	 */
	private Node TextNode(NodeList list) {
		return list.item(0);
	}
	/**
	 * @param list
	 */
	private Node endTextNode(NodeList list) {
		return list.item(list.getLength() - 1);

	}

	/**
	 * @param keyName
	 * @param valueType
	 * @param value
	 */
	private void setAttribute(String keyName, String value) {
		if ("Name".equals(keyName)) {
			_name = value;
		} else if ("Playlist ID".equals(keyName)) {
			_playlistId = Integer.parseInt(value);
		} else if ("Playlist Persistent ID".equals(keyName)) {
			_playlistPersistentId = value;
		}
	}

	public void addSong(Song song) {
		_songs.add(song);

		int index = creatSongNode(_xmlSongsNode);
		setSongInNode(
			_xmlSongsNode.getChildNodes().item(index),
			"" + song.getId());
	}

	/**
	 * @param _xmlSongsNode
	 */
	private int creatSongNode(Node songsNode) {
		if ("0"
			.equals(
				songsNode
					.getChildNodes()
					.item(1)
					.getChildNodes()
					.item(2)
					.getChildNodes()
					.item(0)
					.getNodeValue())) {
			return 1;
		} else {
			Node songTextNode =
				songsNode.getChildNodes().item(0).cloneNode(true);
			Node songElementNode =
				songsNode.getChildNodes().item(1).cloneNode(true);

			songsNode.insertBefore(songTextNode, songsNode.getLastChild());
			songsNode.insertBefore(songElementNode, songsNode.getLastChild());

			return songsNode.getChildNodes().getLength() - 2;
		}
	}
	private void setSongInNode(Node dictNode, String songid) {
		dictNode.getChildNodes().item(1).getChildNodes().item(0).setNodeValue(
			"Track ID");
		dictNode.getChildNodes().item(2).getChildNodes().item(0).setNodeValue(
			"" + songid);
	}
	public void removeSong(int index) {
		LocalXMLSong song = (LocalXMLSong) _songs.get(index);
		int nodeIndex = findIndexInSongsNode(song);
		delSongInNode(_xmlSongsNode.getChildNodes().item(nodeIndex));
		_songs.remove(index);

	}
	private void delSongInNode(Node node) {
		_xmlSongsNode.removeChild(node);

	}
	private int findIndexInSongsNode(Song song) {
		for (int i = 1;
			i < _xmlSongsNode.getChildNodes().getLength() - 1;
			i += 2) {
			int id =
				Integer.parseInt(
					""
						+ _xmlSongsNode
							.getChildNodes()
							.item(i)
							.getChildNodes()
							.item(2)
							.getChildNodes()
							.item(0)
							.getNodeValue());
			if (song.getId() == id);
			return i;
		}
		System.err.println("Not found songId");
		return 0;
	}
	public Song[] getAllSongs() {
		return (Song[]) _songs.toArray();
	}

	public String getName() {
		return _name;
	}
	public int getPlaylistId() {
		return _playlistId;
	}
	public String getPlaylistPersistentId() {
		return _playlistPersistentId;
	}
	public Node getXMLNode() {
		return _xmlNode;
	}
	public Node getXMLSongsNode() {
		return _xmlSongsNode;
	}

	public void setName(String name) {
		_name = name;
		setElement("Name", name);

	}
	public void setPlaylistId(int id) {
		_playlistId = id;
		setElement("Playlist ID", "" + _playlistId);
	}
	public void setPlaylistPersistentId(String persistentId) {
		_playlistPersistentId = persistentId;
		setElement("Playlist Persistent ID", persistentId);
	}
	private void setElement(String string, String nodeValue) {
		NodeList list = _xmlNode.getChildNodes();

		int i = 0;
		if ("Name"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			if ("Name".equals(string)) {
				list.item(i + 2).getChildNodes().item(0).setNodeValue(
					"" + nodeValue);
			}
			i += 3;
		}
		if ("Master"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			i += 3;
		}
		if ("Playlist ID"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			if ("Playlist ID".equals(string)) {
				list.item(i + 2).getChildNodes().item(0).setNodeValue(
					"" + nodeValue);
			}
			i += 3;
		}
		if ("Playlist Persistent ID"
			.equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			if ("Playlist Persistent ID".equals(string)) {
				list.item(i + 2).getChildNodes().item(0).setNodeValue(
					"" + nodeValue);
			}
			i += 3;
		}

	}

	public int getSongSize() {
		return _songs.size();
	}
	public Song getSong(int index) {
		return (LocalXMLSong) _songs.get(index);
	}

	/**
	 * @return
	 */
	public boolean isSmartList() {
		return _isSmartList;
	}

}
