package model.impl.local;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;

import model.api.Song;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Jeong Ingu
 *
 */
public class LocalXMLSong implements Song {

	private int _id;
	private String _name;
	private String _artist;
	private String _album;
	private String _genre;
	private String _kind;
	private long _size;
	private long _totalTime;
	private int _trackNumber;
	private String _year;
	private String _dateModified;
	private String _dateAdded;
	private int _bitRate;
	private int _sampleRate;
	private String _comments;
	private int _playCount;
	private long _playDate;
	private String _playDateUTC;
	private int _rating;
	private String _fileLocation;

	private Node _xmlNode;
	
	private static int NEXT_TRACK_ID = 33;
	
	public static int getNextTrackId(){
		return NEXT_TRACK_ID++;
	}
	
	public LocalXMLSong(Song song, NodeList trackList) {
		createEmptyNode(trackList);
		copySongInfo(song);
		copySongInfoInNode(song);
		
		setIdInNode();
		
	}
	/**
	 * 
	 */
	private void setIdInNode() {
	
		_xmlNode.getChildNodes().item(2).getChildNodes().item(0).setNodeValue("" + _id);
		
	}
	
	/**
	 * @param song
	 */
	private void copySongInfo(Song song) {
		_id = song.getId();
		_name = song.getName();
		_artist = song.getArtist();
		_album = song.getAlbum();
		_genre = song.getGenre();
		_kind = song.getKind();
		_size = song.getSize();
		_totalTime = song.getTotalTime();
		_trackNumber = song.getTrackNumber();
		_year = song.getYear();
		_dateModified = song.getModifiedDate();
		_dateAdded = song.getAddedDate();
		_bitRate = song.getBitRate();
		_sampleRate = song.getSampleRate();
		_comments = song.getComments();
		_playCount = song.getPlayCount();
		_playDate = song.getPlayDate();
		_playDateUTC = song.getPlayDateUTC();
		_rating = song.getRating();
		_fileLocation = song.getLocation();
		
	}
	/**
	 * @param song
	 */
	private void copySongInfoInNode(Song song) {
		
		NodeList list = _xmlNode.getChildNodes();

		int i=0;
			
		
		if("Track ID".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getId());
			
			i+=3;
		}
		if("Name".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getName());
			i+=3;
		}
		if("Artist".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getArtist());
			i+=3;
		}
		if("Album".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getAlbum());
			i+=3;
		}
		if("Genre".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getGenre());
			i+=3;
		}
		if("Kind".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getKind());
			i+=3;
		}
		if("Size".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getSize());
			i+=3;
		}
		if("Total Time".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getTotalTime());
			i+=3;
		}
		if("Year".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getYear());
			i+=3;
		}               
		if("Date Modified".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getModifiedDate());
			i+=3;
		}
		if("Date Added".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getAddedDate());
			i+=3;
			}
		if("Bit Rate".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getBitRate());
			i+=3;
		}
		if("Sample Rate".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getSampleRate());
			i+=3;
		}
		if("Comments".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getComments());
			i+=3;
		}
		if("Play Count".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getPlayCount());
			i+=3;
		}
		if("Play Date".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getPlayDate());
			i+=3;
		}
		if("Play Date UTC".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getPlayDateUTC());
			i+=3;
		}
		if("Rating".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getRating());
			i+=3;
		}
		if("Location".equals(list.item(i + 1).getChildNodes().item(0).getNodeValue())) {
			list.item(i+2).getChildNodes().item(0).setNodeValue("" + song.getLocation());
			i+=3;
		}
		/*
		list.item(2).getChildNodes().item(0).setNodeValue(""+song.getFileFolderCount());
		list.item(2).getChildNodes().item(0).setNodeValue(""+song.getLibraryFolderCount();
		*/

	}
	private void createEmptyNode(NodeList trackList) {
		
		_xmlNode = trackList.item(3).cloneNode(true);
	}

	/**
	 * @param list
	 */
	public LocalXMLSong(Node dictNode) {
		_xmlNode = dictNode;
		NodeList list = dictNode.getChildNodes();
		for (int i = 0; i < list.getLength() - 1; i += 3) {
			String keyName =
				list.item(i + 1).getChildNodes().item(0).getNodeValue();
			String valueType = list.item(i + 2).getNodeName();
			if( valueType.equals("true")) {
				continue;
			}
			String value =
				list.item(i + 2).getChildNodes().item(0).getNodeValue();

			setAttribute(keyName, valueType, value);
		}
		
		_id = getNextTrackId();
		setIdInNode();
		
		
	}

	/**
	 * @param keyName
	 * @param valueType
	 * @param value
	 */
	private void setAttribute(String keyName, String valueType, String value) {
		if ("Track ID".equals(keyName)) {
			_id = Integer.parseInt(""+value);
		} else if ("Name".equals(keyName)) {
			_name = value;
		} else if ("Artist".equals(keyName)) {
			_artist = value;
		} else if ("Album".equals(keyName)) {
			_album = value;
		} else if ("Genre".equals(keyName)) {
			_genre = value;
		} else if ("Kind".equals(keyName)) {
			_kind = value;
		} else if ("Size".equals(keyName)) {
			_size = Long.parseLong(value);
		} else if ("Total Time".equals(keyName)) {
			_totalTime = Long.parseLong(value);
		} else if ("Year".equals(keyName)) {
			_year = value;
		} else if ("Date Modified".equals(keyName)) {
			_dateModified = value;
		} else if ("Date Added".equals(keyName)) {
			_dateAdded = value;
		} else if ("Bit Rate".equals(keyName)) {
			_bitRate = Integer.parseInt(value);
		} else if ("Sample Rate".equals(keyName)) {
			_sampleRate = Integer.parseInt(value);
		} else if ("Comments".equals(keyName)) {
			_comments = value;
		} else if ("Play Count".equals(keyName)) {
			_playCount = Integer.parseInt(value);
		} else if ("Play Date".equals(keyName)) {
			_playDate = Long.parseLong(value);
		} else if ("Play Date UTC".equals(keyName)) {
			_playDateUTC = value;
		} else if ("Rating".equals(keyName)) {
			_rating = Integer.parseInt(value);
		} else if ("Location".equals(keyName)) {

			try {
				_fileLocation = URLDecoder.decode(value, "UTF-8");
			} catch (UnsupportedEncodingException e) {

				e.printStackTrace();
			}

		}

		//
		// URL  java.net.URLEncoder URLDecoder Ͽ óҰ
		// 

	}

	public int getId() {
		return _id;
	}
	public String getName() {

		return _name;
	}
	public String getArtist() {

		return _artist;
	}
	public String getAlbum() {

		return _album;
	}
	public String getGenre() {
		return _genre;
	}
	public String getKind() {
		return _kind;
	}
	public long getSize() {
		return _size;
	}
	public long getTotalTime() {

		return _totalTime;
	}
	public int getTrackNumber() {
		return _trackNumber;
	}
	public String getYear() {
		return _year;
	}
	public String getModifiedDate() {

		return _dateModified;
	}

	public String getAddedDate() {

		return _dateAdded;
	}

	public int getBitRate() {

		return _bitRate;
	}
	public int getSampleRate() {

		return _sampleRate;
	}

	public String getComments() {

		return _comments;

	}

	public int getPlayCount() {
		return _playCount;
	}
	public long getPlayDate() {
		return _playDate;
	}
	public String getPlayDateUTC() {
		return _playDateUTC;
	}
	public int getRating() {
		return _rating;
	}
	public String getLocation() {

		return _fileLocation;
	}

	public InputStream getMusicStream() throws IOException {
		URL fileUrl = new URL(getLocation());
		return fileUrl.openStream();
	}
	public Node getXMLNode() {
		return _xmlNode;
		
	}
}
